/* ------------------------------------------------------------------------- */
/*                                                                           */
/* Copyright (c) 2003 The Open Group                                         */
/*                                                                           */
/* Permission is hereby granted, free of charge, to any person obtaining a   */
/* copy of this software (the "Software"), to deal in the Software without   */
/* restriction, including without limitation the rights to use, copy,        */
/* modify, merge, publish, distribute, sublicense, and/or sell copies of     */
/* the Software, and to permit persons to whom the Software is furnished     */
/* to do so, subject to the following conditions:                            */
/*                                                                           */
/* The above copyright notice and this permission notice shall be included   */
/* in all copies or substantial portions of the Software.                    */
/*                                                                           */
/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS   */
/* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                */
/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.    */
/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY      */
/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT */
/* OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR  */
/* THE USE OR OTHER DEALINGS IN THE SOFTWARE.                                */
/*                                                                           */
/* ------------------------------------------------------------------------- */
/*                                                                           */
/* File revision information                                                 */
/*                                                                           */
/* $Source: /mnt/cvslocal/sdk4/java/org/opengroup/arm40/tranreport/ArmSystemAddress.java,v $ */
/* $Revision: 1.1 $ */
/* $Date: 2005/03/14 15:14:53 $ */
/*                                                                           */
/* ------------------------------------------------------------------------- */

package org.opengroup.arm40.tranreport;

import org.opengroup.arm40.transaction.ArmToken;
import org.opengroup.arm40.transaction.ArmID;

/**
 * Encapsulates the network addressing information for a system.
 * It may be used with {@link ArmTranReport} if the reported 
 * transaction executed on a different system.
 * <ul>
 * <li>
 * <b>Address</b>. The system address is the network name 
 * or address of the system, as it would be sent in a data frame 
 * across a network in network byte order.
 * </li>
 * <li>
 * <b>Format</b>. The format of the system address, such as an 
 * SNA address or a hostname. See the description of format
 * constants defined in this interface for available formats
 * and the mapping of the address bytes supplied. 
 * <br/>Note that all positive
 * integer values not defined as constant are reserved for
 * future definitions. Currently, this pplies to values 9:32767.
 * All negative values are available for implementation-defined 
 * use.
 * </li>
 * <li>
 * <b>ID</b>. An optional 16-byte ID associated with the format 
 * and address, if any. The ID value is bound to a unique combination 
 * of the format and address. When provided, the ID may be used 
 * as a concise alias for the unique combination. It may be 
 * <code>null</code>.
 * <p/>
 * The fields are set using the <code>newArmSystemAddress()</code> method
 * of <code>ArmTranReportFactory</code> or the 
 * <code>getArmSystemAddress()</code> method of <code>ArmSystem</code>. 
 * There are no setter methods for the individual fields. The object is 
 * immutable.
 * <p/>
 * Implementations of this interface should also override 
 * <code>equals()</code> and <code>hashCode()</code> from 
 * <code>java.lang.Object</code>.
 * <p/>
 * <code>equals(Object obj)</code>, a method inherited from 
 * <code>java.lang.Object</code>, returns <code>true</code> if the internal 
 * data is byte-for-byte identical in two objects. For example, 
 * <code>a.equals(b)</code> returns true if and only if:
 * <ul>
 * <li>
 * Both <code>a</code> and <code>b</code> implement <code>ArmSystemAddress</code>.
 * </li>
 * <li>
 * The inherited methods <code>a.getBytes()</code> and 
 * <code>b.getBytes()</code> would return byte arrays of identical lengths 
 * and contents.
 * </li>
 * <li>
 * <code>a.getFormat()</code> and <code>b.getFormat()</code> would return 
 * identical values.
 * </li>
 * </ul>
 * 
 * <p/>Objects implementing this interface are created using
 * {@link ArmTranReportFactory#newArmSystemAddress}.
 * 
 * @author ARM Working Group of The Open Group
 */
public interface ArmSystemAddress extends ArmToken {

// public constants

    /**
     * a hostname (characters, not null-terminated).
     * <p>Bytes 0:?? = hostname
     */
    public static final short FORMAT_HOSTNAME = 7;

    /**
     * an IPv4 address.
     * <p/>Bytes 0:3 = 4-byte IP address.
     */
    public static final short FORMAT_IPV4 = 1;

    /**
     * an IPv4 address and port number.
     * <p/>Bytes 0:3 = 4-byte IP address<br/>
     * Bytes 4:5 = 2-byte IP port number
     */
    public static final short FORMAT_IPV4PORT = 2;

    /**
     * an IPv6 address.
     * <p/>Bytes 0:15 = 16-byte IP address
     */
    public static final short FORMAT_IPV6 = 3;

    /**
     * an IPv6 address and port number.
     * <p/>Bytes 0:15 = 16-byte IP address<br/>
     * Bytes 16:17 = 2-byte IP port number
     */
    public static final short FORMAT_IPV6PORT = 4;

    /**
     * an SNA address.
     * <p/>Bytes 0:7 = EBCDIC-encoded network ID<br/>
     * Bytes 8:15 = EBCDIC-encoded network accessible unit 
     * (control point or LU)
     */
    public static final short FORMAT_SNA = 5;

    /**
     * an X.25 address.
     * <p/>Bytes 0:15 = The X.25 address (also referred to as an X.121 
     * address). This is up to 16 ASCII character digits ranging from 0-9.
     */
    public static final short FORMAT_X25 = 6;

    /**
     * a Universally-unique ID.
     * <p/>Bytes 0:15 = UUID in binary.
     * This is useful for applications that define their system by a UUID 
     * rather than a network address or hostname or some other address form.
     */
    public static final short FORMAT_UUID = 8;

// No Public Constructors

    /**
     * @return a byte array containing the address. The returned value is the
     *         same value passed to the <code>newArmSystemAddress()</code>
     *         method of <code>ArmTranReportFactory</code>.  
     */
    public byte[] getAddress();

    /**
     * @return a short containing the format. The returned value is the
     *         same value passed to the <code>newArmSystemAddress()</code>
     *         method of <code>ArmTranReportFactory</code>.
     */
    public short getFormat();

    /**
     * @return the optional 16-byte ID associated with the format and address,
     *         if any. The returned value, which could be null, is the
     *         same value passed to the <code>newArmSystemAddress()</code>
     *         method of <code>ArmTranReportFactory</code>.
     */
    public ArmID getID();
}

