/* ------------------------------------------------------------------------- */
/*                                                                           */
/* Copyright (c) 2003 The Open Group                                         */
/*                                                                           */
/* Permission is hereby granted, free of charge, to any person obtaining a   */
/* copy of this software (the "Software"), to deal in the Software without   */
/* restriction, including without limitation the rights to use, copy,        */
/* modify, merge, publish, distribute, sublicense, and/or sell copies of     */
/* the Software, and to permit persons to whom the Software is furnished     */
/* to do so, subject to the following conditions:                            */
/*                                                                           */
/* The above copyright notice and this permission notice shall be included   */
/* in all copies or substantial portions of the Software.                    */
/*                                                                           */
/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS   */
/* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                */
/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.    */
/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY      */
/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT */
/* OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR  */
/* THE USE OR OTHER DEALINGS IN THE SOFTWARE.                                */
/*                                                                           */
/* ------------------------------------------------------------------------- */
/*                                                                           */
/* File revision information                                                 */
/*                                                                           */
/* $Source: /mnt/cvslocal/sdk4/java/org/opengroup/arm40/tranreport/ArmTranReport.java,v $ */
/* $Revision: 1.1 $ */
/* $Date: 2005/03/14 15:14:53 $ */
/*                                                                           */
/* ------------------------------------------------------------------------- */

package org.opengroup.arm40.tranreport;

import org.opengroup.arm40.transaction.ArmInterface;
import org.opengroup.arm40.transaction.ArmCorrelator;
import org.opengroup.arm40.transaction.ArmApplication;
import org.opengroup.arm40.transaction.ArmTransactionDefinition;
import org.opengroup.arm40.transaction.ArmUser;

/**
 * Transaction instance for application-based and/or remote time measurement.
 * <p/> 
 * <code>ArmTranReport</code> is similar to <code>ArmTransaction</code>. 
 * Both are used to provide data about executing transactions. Instances 
 * of both are created based on metadata represented by an 
 * <code>ArmTransactionDefinition</code>, which in turn is scoped by an 
 * application definition. Both are scoped by a running application 
 * instance, represented by ArmApplication. 
 * There are two fundamental differences:
 * <ul>
 * <li>
 * With <code>ArmTransaction</code>, the response time is measured based 
 * on <code>start()</code> and <code>stop()</code> events. With 
 * <code>ArmTranReport</code>, the application measures the response time,
 * and reports it with a single <code>report()</code> event.
 * </li>
 * <li>
 * With <code>ArmTransaction</code>, the transaction always executes on 
 * the local system in the same JVM (Java Virtual Machine). With 
 * <code>ArmTranReport</code>, the transaction may execute in the same 
 * JVM, in a different JVM on the same system, or on a different system.
 * </li>
 * </ul>
 * When executing in the same JVM, the <code>ArmTranReport</code> object 
 * is created with an <code>ArmApplication</code>. When executing in a 
 * different JVM on the same system or on a different system, the 
 * <code>ArmTranReport</code> object is created with an 
 * <code>ArmApplicationRemote</code> (a subclass of 
 * <code>ArmApplication</code>).
 * <p/>
 * The two key methods of ArmTranReport are {@link #generateCorrelator()}
 * and {@link #report}.
 * <ul>
 * <li>
 * <code>generateCorrelator()</code> generates a new correlator using the
 * immutable data set in the factory method and the current property values
 * set by the four setter methods. It is assumed that 
 * <code>generateCorrelator()</code> is executed zero or once per transaction
 * instance. The practical ramification is that the method implementation 
 * will update its internal state to have a unique identifier for an 
 * instance (the equivalent of <code>ArmTransaction</code>'s start handle).
 * <code>getCorrelator()</code> returns the most recently generated 
 * <code>ArmCorrelator</code> or null, if <code>generateCorrelator()</code>
 * has never been executed.
 * </li>
 * <li>
 * <code>report()</code> is used to provide measurements about a completed
 * transaction. There are two forms. Both provide the status (one of the 
 * <code>STATUS_*</code> constants in <code>ArmConstants</code>) and the 
 * response time, measured in nanoseconds.
 * One also provides a stop time in the form of milliseconds
 * since January 1, 1970, which is the same format returned by 
 * <code>java.lang.System.currentTimeMillis()</code>. If a stop time is not
 * provided, or a stop time of -1 (<code>USE_CURRENT_TIME</code>) is provided,
 * the ARM implementation substitutes the current time; that is, the time 
 * when the <code>report()</code> method executes. The optional form that 
 * takes a string is a way for an application to provide additional diagnostic
 * details when the status is something other than <code>STATUS_GOOD</code>.
 * </li>
 * </ul>
 * As noted above, <code>generateCorrelator()</code> updates the internal 
 * state for a new transaction instance. The first time <code>report()</code>
 * executes after <code>generateCorrelator()</code>, <code>report()</code>
 * will not update the internal state for a new transaction instance; 
 * it will use the instance identifier from the 
 * <code>generateCorrelator()</code>. If <code>report()</code> executes 
 * twice in succession, or if <code>generateCorrelator()</code> has never 
 * been executed, <code>report()</code> will update the internal state for 
 * a new transaction instance. Summarizing, there are two patterns:
 * <ul>
 * <li>
 * If correlators for the current transaction are not requested,
 * <code>generateCorrelator()</code> is not used. <code>report()</code> is
 * executed after each instance completes, and each time it generates a new
 *  instance identifier, like a start handle.
 * </li>
 * <li>
 * If correlators for the current transaction are requested, 
 * <code>generateCorrelator()</code> and <code>report()</code> are used in 
 * pairs. First <code>generateCorrelator()</code> establishes the transaction
 * instance identifiers, as well as creating a correlator. This correlator
 * is sent to downstream transactions. After the downstream transactions
 * complete, and the current instance completes, <code>report()</code> 
 * provides the measurements. In this case <code>report()</code> does not 
 * update the transaction instance identifier.
 * </li>
 * </ul>
 * In addition to the identity properties from <code>ArmApplication</code>
 * and <code>ArmTransactionDefinition</code>, there are four optional 
 * setter methods to establish additional instance-level context. 
 * They can be used at any time to update the attribute within the object.
 * The only time the properties are meaningful is when 
 * <code>generateCorrelator()</code> or <code>report()</code> executes. 
 * At the moment either method executes, the current values are used, any 
 * or all of which may be null. See the description provided for the 
 * individual methods below.
 * 
 * <p/>Objects implementing this interface are created using
 * {@link ArmTranReportFactory#newArmTranReport}.
 * 
 * @author ARM Working Group of The Open Group
 */
public interface ArmTranReport extends ArmInterface {

// No Public Constructors

// Public Instance Methods

    /**
     * Generates a new correlator for the transaction instance. See
     * the discussion of this method in the interface description above.
     * @return a new <code>ArmCorrelator</code>
     */
    public ArmCorrelator generateCorrelator();

    /**
     * @return the <code>ArmApplication</code> used in the factory method 
     *         to create this instance of <code>ArmTranReport</code>.
     */
    public ArmApplication getApplication();

    /**
     * gets the context URI value.
     * @return the context URI value, or <code>null</code>
     */
    public String getContextURIValue();

    /**
     * gets a context property value. See the description of
     * {@link #setContextValue(int, String)}.
     * @param index index into the array aof context properties.
     * @return the value at <code>index</code>, or <code>null</code>.
     */
    public String getContextValue(int index);

    /**
     * gets the current correlator for this transaction instance. See
     * the discussion in the interface description above for when
     * the state of the instance affecting the correlator is updated.
     * @return an <code>ArmCorrelator</code> for this transcation instance.
     */
    public ArmCorrelator getCorrelator();

    /**
     * gets the correlator of the parent transaction, if any.
     * @return the last value set with {@link #setParentCorrelator}.
     *         The vaue may be <code>null</code>. 
     */
    public ArmCorrelator getParentCorrelator();

	/**
	 * returns the last response time set on a <code>report()</code> method. 
	 * If <code>report()</code> has never executed, the return value
	 * is implementation dependent.
	 * @return the response time as of the last <code>report()</code> call. 
	 */
    public long getResponseTime();

	/**
	 * returns the last status value set on a <code>report()</code> method. 
	 * If <code>report()</code> has never executed, it returns 
	 * <code>STATUS_INVALID</code>.
	 * @return one of the status values defined in 
	 *         {@link org.opengroup.arm40.transaction.ArmConstants}. 
	 */
    public int getStatus();

    /**
     * @return the <code>ArmTransactionDefinition</code> used in the 
     *         factory method to create this instance of 
     *         <code>ArmTranReport</code>.
     */
    public ArmTransactionDefinition getDefinition();

    /**
     * @return the user associated with this transaction instance, if
     *         set with {@link #setUser(ArmUser)}, or <code>null</code>.
     */
    public ArmUser getUser();

    /**
     * provide measurements about a completed transaction. See
     * the discussion of this method in the interface description above.
     * 
     * @param status transaction status. 
     *               See {@link org.opengroup.arm40.transaction.ArmConstants}.
     * @param respTimeNanos response time (in nanoseconds)
     * @return 0 on sucess; otherwise, a non-zero error code is returned (as
	 *         specified in 
	 *         {@link org.opengroup.arm40.transaction.ArmInterface}).
     */
    public int report(int status, long respTimeNanos);

    /**
     * provides measurements about a completed transaction. See
     * the discussion of this method in the interface description above.
     * 
     * @param status transaction status. 
     *               See {@link org.opengroup.arm40.transaction.ArmConstants}.
     * @param respTimeNanos response time (nanoseconds)
     * @param stopTime stop time (milliseconds since January 1, 1970) 
     * @return 0 on sucess; otherwise, a non-zero error code is returned (as
	 *         specified in 
	 *         {@link org.opengroup.arm40.transaction.ArmInterface}).
     */
    public int report(int status, long respTimeNanos, long stopTime);

    /**
     * provide measurements about a completed transaction. See
     * the discussion of this method in the interface description above.
     * 
     * @param status transaction status. 
     *               See {@link org.opengroup.arm40.transaction.ArmConstants}.
     * @param respTimeNanos response time (in nanoseconds)
     * @param diagnosticDetail additional details, used when status other than
     *                         <code>ArmConstants.STATUS_GOOD</code>
     * @return 0 on sucess; otherwise, a non-zero error code is returned (as
	 *         specified in 
	 *         {@link org.opengroup.arm40.transaction.ArmInterface}).
     */
    public int report(int status, long respTimeNanos, String diagnosticDetail);

    /**
     * provide measurements about a completed transaction. See
     * the discussion of this method in the interface description above.
     * 
     * @param status transaction status. 
     *               See {@link org.opengroup.arm40.transaction.ArmConstants}.
     * @param respTimeNanos response time (in nanoseconds)
     * @param stopTime stop time (milliseconds since January 1, 1970)
     * @param diagnosticDetail additional details, used when status other than
     *                         <code>ArmConstants.STATUS_GOOD</code> 
     * @return 0 on sucess; otherwise, a non-zero error code is returned (as
	 *         specified in 
	 *         {@link org.opengroup.arm40.transaction.ArmInterface}).
     */
    public int report(int status, long respTimeNanos, long stopTime, String diagnosticDetail);

    /**
     * sets the URI context value.
     * @param value the new URI context value.
     * @return 0 on sucess; otherwise, a non-zero error code is returned (as
	 *         specified in 
	 *         {@link org.opengroup.arm40.transaction.ArmInterface}).
     */
    public int setContextURIValue(String value);

    /**
     * sets a context property value.
     * This method sets one of the maximum 20 context properties that may 
     * change for each transaction instance. The context property name 
     * at the specified array index must have been set to a 
     * non-<code>null</code> value when the 
     * <code>ArmTransactionDefinition</code> object was created. 
     * If the name is <code>null</code>, the value will be set to 
     * <code>null</code>.
     * @param index index into the array aof context properties.
     * @param value the new context value.
     * @return 0 on sucess; otherwise, a non-zero error code is returned (as
	 *         specified in 
	 *         {@link org.opengroup.arm40.transaction.ArmInterface}).
     */
    public int setContextValue(int index, String value);

    /**
     * sets the correlator of the parent transaction.
     * @param parent an {@link org.opengroup.arm40.transaction.ArmCorrelator}
     *               object referring to the parent transaction, or 
     *               <code>null</code>.
     * @return 0 on sucess; otherwise, a non-zero error code is returned (as
	 *         specified in 
	 *         {@link org.opengroup.arm40.transaction.ArmInterface}).
     */
    public int setParentCorrelator(ArmCorrelator parent);

    /**
     * associates a user, represented by an instance of <code>ArmUser</code>, 
     * to the ArmTranReport instance. This user is assumed to be the user for 
     * all <code>start()</code>/<code>stop()</code> pairs until the 
     * association is changed or cleared. 
     * <code>setUser(null)</code> clears any existing association to an 
     * <code>ArmUser</code>.
     * @param user an <code>ArmUser</code>, or <code>null</code>
     * @return 0 on sucess; otherwise, a non-zero error code is returned (as
	 *         specified in 
	 *         {@link org.opengroup.arm40.transaction.ArmInterface}).
     */
    public int setUser(ArmUser user);
}
