/* ------------------------------------------------------------------------- */
/*                                                                           */
/* Copyright (c) 2003 The Open Group                                         */
/*                                                                           */
/* Permission is hereby granted, free of charge, to any person obtaining a   */
/* copy of this software (the "Software"), to deal in the Software without   */
/* restriction, including without limitation the rights to use, copy,        */
/* modify, merge, publish, distribute, sublicense, and/or sell copies of     */
/* the Software, and to permit persons to whom the Software is furnished     */
/* to do so, subject to the following conditions:                            */
/*                                                                           */
/* The above copyright notice and this permission notice shall be included   */
/* in all copies or substantial portions of the Software.                    */
/*                                                                           */
/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS   */
/* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                */
/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.    */
/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY      */
/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT */
/* OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR  */
/* THE USE OR OTHER DEALINGS IN THE SOFTWARE.                                */
/*                                                                           */
/* ------------------------------------------------------------------------- */
/*                                                                           */
/* File revision information                                                 */
/*                                                                           */
/* $Source: /mnt/cvslocal/sdk4/java/org/opengroup/arm40/transaction/ArmApplication.java,v $    */
/* $Revision: 1.1 $  */
/* $Date: 2005/03/14 15:14:53 $      */
/*                                                                           */
/* ------------------------------------------------------------------------- */

package org.opengroup.arm40.transaction;

/**
 * Represents an instance of an executing application. 
 * 
 * It provides an anchor point for associating {@link ArmTransaction}
 * objects with the application instance.
 * It has the following attributes, all of which are immutable: 
 * <ul>
 * <li>
 * <b>Application definition</b>. The metadata common to all instances of 
 * this application. The value must not be <code>null</code>.
 * </li>
 * <li>
 * <b>Group</b> (optional). A group is a set of application instances 
 * that are treated as a group for some aspects of management, such as 
 * workload balancing (work could be routed to any instance of the group). 
 * The maximum length is 255 characters. The value may be <code>null</code>.
 * A non-<code>null</code> value should not contain trailing blank 
 * characters or consist of only blank characters.
 * </li>
 * <li>
 * <b>Instance</b> (optional). The instance could be used to distinguish 
 * between instances of the same application. ARM does not require 
 * that this field be used or that it be unique, though its use is suggested.
 * The maximum length is 255 characters. The value may be <code>null</code>.
 * A non-<code>null</code> value should not contain trailing blank 
 * characters or consist of only blank characters.
 * </li>
 * <li>
 * <b>Context values</b> (optional). The "value" part of (name,value) 
 * properties that may vary per application instance. The "name" part 
 * is available via 
 * <code>getDefinition().getIdentityProperties().getContextName()</code>.
 * The values are position-sensitive - they match the position in the 
 * referenced context name array. The context property name at the specified 
 * array index must have been set to a non-<code>null</code> value when the 
 * {@link ArmApplicationDefinition} object was created. If the name is 
 * <code>null</code> or a zero-length string, both the name and value are 
 * ignored. If the value is <code>null</code> or a zero-length string, 
 * the meaning is that there is no value for this instance. Names should not 
 * contain trailing blank characters or consist of only blank characters. 
 * <p/>
 * Note that both <code>ArmApplication</code> and 
 * <code>ArmTransaction</code> have context values that may be 
 * unique for each application and transaction instance, respectively. 
 * However, the mechanisms for setting the context values are different. 
 * In <code>ArmApplication</code>, they are set in the factory method and 
 * are immutable afterwards. In <code>ArmTransaction</code>, they are set at 
 * any time and processed each time a <code>start()</code> executes.
 * <p/> 
 * The reason for the difference is that <code>ArmApplication</code> objects
 * are not reused. One object is created per application instance. 
 * If another instance starts, a new <code>ArmApplication</code> is 
 * created. This is a reasonable design pattern because applications 
 * are long-lived. In most cases, once an application starts, it continues 
 * to execute for a long time, so the cost of creating the 
 * <code>ArmApplication</code> object and then garbage collecting it is not 
 * significant.
 * <p/>
 * Transactions, on the other hand, are often very short-lived, with 
 * response times that are often measured in milliseconds. Creating a 
 * new <code>ArmTransaction</code> object for each transaction instance, 
 * and then garbage collecting it afterwards, would result in unacceptable 
 * and unnecessary overhead. Instead, an <code>ArmTransaction</code> object 
 * is created and then reused over and over. Each executing transaction 
 * is represented by the paired <code>start()</code>/<code>stop()</code> 
 * methods. For this reason, context values are set using setter methods.
 * </li>
 * </ul>
 * <code>end()</code> indicates that the application instance has 
 * halted. After <code>end()</code> executes, the application instance 
 * may not call any other method of the <code>ArmApplication</code>
 * object, it may not use any reference to the <code>ArmApplication</code>
 * object, nor may it call any method of any object created using a 
 * reference to the <code>ArmApplication</code> object (e.g., creating 
 * an instance of <code>ArmTransaction</code> using the 
 * <code>newArmTransaction()</code> method of 
 * <code>ArmTransactionFactory</code>). Any transactions that are currently
 * in-process [<code>start()</code> executed but <code>stop()</code> not 
 * executed] will be discarded by implicitly executing the 
 * <code>ArmTransaction reset()</code> method. The ARM implementation 
 * should protect itself against a poorly behaved application that does 
 * not respect the specification. 
 * <p/>
 * Objects implementing this interface are created using 
 * {@link ArmTransactionFactory#newArmApplication}.
 * 
 * @author ARM Working Group of The Open Group
 */

public interface ArmApplication extends ArmInterface {

// No Public Constructors

// Public Instance Methods

    /**
     * indicates that the application instance has halted. See
     * comments in the interface description above.
     * 
     * @return 0 on sucess; otherwise, a non-zero error code is returned 
     * (as specified in {@link org.opengroup.arm40.transaction.ArmInterface}).
     */
    public int end();
        
    /**
     * retrieves a context property value.
     * 
     * @param index the index into the context properties array. 
     * @return The context value at the specified index, or 
     * <code>null</code>. See comments in the interface description above.
     */
    public String getContextValue(int index);
        
    /**
     * @return the metadata describing this application.
     */
    public ArmApplicationDefinition getDefinition();
        
    /**
     * @return the group name this application is associated with,
     * or <code>null</code>. See comments in the interface description above.
     */
    public String getGroup();

    /**
     * @return the instance name attributed to this application,
     * or <code>null</code>. See comments in the interface description above.
     */
    public String getInstance();
}

