/* ------------------------------------------------------------------------- */
/*                                                                           */
/* Copyright (c) 2003 The Open Group                                         */
/*                                                                           */
/* Permission is hereby granted, free of charge, to any person obtaining a   */
/* copy of this software (the "Software"), to deal in the Software without   */
/* restriction, including without limitation the rights to use, copy,        */
/* modify, merge, publish, distribute, sublicense, and/or sell copies of     */
/* the Software, and to permit persons to whom the Software is furnished     */
/* to do so, subject to the following conditions:                            */
/*                                                                           */
/* The above copyright notice and this permission notice shall be included   */
/* in all copies or substantial portions of the Software.                    */
/*                                                                           */
/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS   */
/* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                */
/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.    */
/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY      */
/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT */
/* OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR  */
/* THE USE OR OTHER DEALINGS IN THE SOFTWARE.                                */
/*                                                                           */
/* ------------------------------------------------------------------------- */
/*                                                                           */
/* File revision information                                                 */
/*                                                                           */
/* $Source: /mnt/cvslocal/sdk4/java/org/opengroup/arm40/transaction/ArmToken.java,v $ */
/* $Revision: 1.1 $ */
/* $Date: 2005/03/14 15:14:53 $ */
/*                                                                           */
/* ------------------------------------------------------------------------- */
/*                                                                           */
/* ArmToken                                                                  */
/*                                                                           */
/* ArmToken is an abstract interface which is a superclass of ArmCorrelator, */
/* ArmSystemAddress (in the tranreport package), and ArmID, expressing the   */
/* common part of their interfaces.                                          */
/*                                                                           */
/* ArmToken cannot be instantiated.                                          */
/*                                                                           */
/* ------------------------------------------------------------------------- */

package org.opengroup.arm40.transaction;

/**
 * An abstract interface serving as a superclass of ArmCorrelator, 
 * ArmSystemAddress, and ArmID.
 * <p/>
 * <code>ArmToken</code> expresses the common part of the above interfaces.
 * It is abstract in the sense that any <code>ArmToken</code> object 
 * returned by any method in this specification satisfies one of the 
 * subclass interfaces. Objects of these identify particular entities. 
 * These objects contain a byte array data token, plus optionally other 
 * identifying data, which together comprise the value of the object. 
 * The data token is always immutable. Subclasses of <code>ArmToken</code>
 * can be thought of as wrappers around the data token. 
 * <p/>
 * To make it possible to compare the values of these tokens, and to 
 * use these tokens as hashkeys (so that the user can associate data with 
 * a particular token), this specification requires that any 
 * class implementing any of these types override the methods
 * {@link java.lang.Object#equals(java.lang.Object)} and
 * {@link java.lang.Object#hashCode()}. 
 * The behavior of these methods must be the following.
 * <p/>
 * <code>a.equals(b)</code> returns <code>true</code> if the 
 * <code>ArmToken<code> objects <code>a</code> and <code>b</code> have the
 * same value (internal data is byte-for-byte identical in two objects). 
 * That is, <code>a.equals(b)</code> is true if and only if: 
 * <ul>
 * <li>
 * <code>a</code> and <code>b</code> implement the same interface 
 * <code>ArmCorrelator</code>, <code>ArmSystemAddress</code>, 
 * or <code>ArmID</code>.
 * </li>
 * <li>
 * <code>a.getBytes()</code> and <code>b.getBytes()</code> would return 
 * byte arrays of identical lengths and contents.
 * </li>
 * <li>
 * If a subclass of <code>ArmToken</code> defines other data values 
 * (specifically, <code>ArmSystemAddress</code> defines a short field 
 * named <code>format</code>), these other data values are all identical.
 * </li>
 * </ul>
 * If <code>a.equals(b)==true</code>, <code>a.hashCode()</code> and 
 * <code>b.hashCode()</code> will return the same value. The
 * <code>hashCode()</code> value is implementation-defined. In other words, 
 * hashcode values are not necessarily portable. A hashcode generated on 
 * one system by one implementation may not equal a hashcode value generated 
 * on another system by a different implementation, even if 
 * <code>a.equals(b)==true</code>. 
 * 
 * @author ARM Working Group of The Open Group
 */

public interface ArmToken extends ArmInterface {

// No Public Constructors

// Public Instance Methods
// (Subclasses should also override equals() and hashCode() 
// from java.lang.Object.)

    /**
     * Copies the token to a byte array that is already allocated.
     * 
     * @param dest destination byte array. Its length must be greater
     * than or equal to the length of the token's byte array.
     * 
     * @return <code>true</code> if the operation was successful, 
     * <code>false</code> otherwise. If <code>false</code>, the contents of 
     * the target array are undetermined. The most likely errors are an 
     * attempt to copy into a null pointer or into an array that is not 
     * long enough to hold the entire token.
     */
    public boolean copyBytes(byte[] dest);

    /**
     * Copies the token to a byte array that is already allocated.
     * 
     * @param dest destination byte array. <code>(dest.length-offset)</code>
     * must be greater than or equal to the length of the token's byte array.
     * 
     * @param offset offset in <code>dest</code> to copying at.
     * 
     * @return <code>true</code> if the operation was successful, 
     * <code>false</code> otherwise. If <code>false</code>, the contents of 
     * the target array are undetermined. The most likely errors are an 
     * attempt to copy into a null pointer or into an array that is not 
     * long enough to hold the entire token.
     */
    public boolean copyBytes(byte[] dest, int offset);

    /**
     * Returns a newly allocated byte array into which the token is copied.
     * <p/>
     * This is equivalent to creating a byte array of length 
     * <code>getLength()</code> and then executing <code>copyBytes()</code>
     * into the new array. The ARM implementation would typically not keep 
     * a reference to the array, because that would interfere with garbage 
     * collection.
     * 
     * @return a byte array containing a copy of the token's byte array part. 
     */
    public byte[] getBytes();

    /**
     * Gets the length of the byte array part.
     * @return the size of the byte array part of the token.
     */
    public int getLength();
}
