/* ------------------------------------------------------------------------- */
/*                                                                           */
/* Copyright (c) 2003 The Open Group                                         */
/*                                                                           */
/* Permission is hereby granted, free of charge, to any person obtaining a   */
/* copy of this software (the "Software"), to deal in the Software without   */
/* restriction, including without limitation the rights to use, copy,        */
/* modify, merge, publish, distribute, sublicense, and/or sell copies of     */
/* the Software, and to permit persons to whom the Software is furnished     */
/* to do so, subject to the following conditions:                            */
/*                                                                           */
/* The above copyright notice and this permission notice shall be included   */
/* in all copies or substantial portions of the Software.                    */
/*                                                                           */
/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS   */
/* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                */
/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.    */
/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY      */
/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT */
/* OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR  */
/* THE USE OR OTHER DEALINGS IN THE SOFTWARE.                                */
/*                                                                           */
/* ------------------------------------------------------------------------- */
/*                                                                           */
/* File revision information                                                 */
/*                                                                           */
/* $Source: /mnt/cvslocal/sdk4/java/org/opengroup/arm40/transaction/ArmTransaction.java,v $ */
/* $Revision: 1.1 $ */
/* $Date: 2005/03/14 15:14:53 $ */
/*                                                                           */
/* ------------------------------------------------------------------------- */

package org.opengroup.arm40.transaction;

/**
 * The unit of work representation central to the ARM model. <p/>
 * 
 * For most applications, this is the most important of all the ARM classes, and
 * the most frequently used. Many applications operate only on
 * <code>ArmTransaction</code> objects after some initialization (using
 * <code>ArmTransactionFactory</code>,<code>ArmApplicationDefinition</code>,
 * <code>ArmApplication</code>, and <code>ArmTransactionDefinition</code>).
 * Instances of <code>ArmTransaction</code> represent transactions when they
 * execute. A "transaction" is any unit of work that has a clearly understood
 * beginning and ending point, and which begins and ends in the same Java&copy;
 * Virtual Machine instance. Examples include a remote procedure call, a
 * database transaction, and a batch job. It is not necessary that an ARM
 * transaction implement robust functions such as commit and rollback. <p/>The
 * application creates as many instances as it needs. This will typically be at
 * least as many as the number of transactions that can be executing
 * simultaneously. An application may create a pool of
 * <code>ArmTransaction</code> objects, take one from the pool to use when a
 * transaction starts, and put it back in the pool after the transaction ends
 * for later reuse. Another strategy is to create one instance of each type per
 * thread, which eliminates the need to manage the pool, handle synchronization
 * if the pool is depleted, etc. <p/>The metadata common to all instances is
 * contained in the <code>ArmTransactionDefinition</code> used to create the
 * object. Each transaction is scoped by an application instance, represented by
 * <code>ArmApplication</code>.<p/>The most frequently (and often the only)
 * used methods are <code>start()</code>,<code>getCorrelator()</code>,
 * and <code>stop()</code>. A typical sequence is as follows:
 * <ul>
 * <li>Just prior to executing a transaction, such as a remote procedure call,
 * call <code>start()</code> to signal to ARM that the measurable transaction
 * is beginning. <code>start()</code> causes ARM to capture the current time.
 * If a correlation token was received when your program was invoked, pass it as
 * a parameter on the <code>start()</code>.</li>
 * <li>Just after executing <code>start()</code>, and just prior to
 * executing the transaction, call <code>getCorrelator()</code> to get a
 * correlation token that can be sent along with the other transaction
 * parameters to the receiver. Not all programs use correlators, but their use
 * is highly recommended, because without them, it is usually impossible to
 * drill down to understand the components of a transaction, where they
 * executed, what resources they used, etc.</li>
 * <li>Execute the transaction (e.g., make the remote procedure call).</li>
 * <li>As soon as it ends, call <code>stop()</code>, passing a status to
 * indicate whether the transaction succeeded. <code>stop()</code> causes ARM
 * to capture the current time. The response time is determined by calculating
 * the duration between the <code>start()</code> and <code>stop()</code>
 * events.</li>
 * </ul>
 * See the description of the individual methods below for details about each
 * method. All methods that return an <code>int</code> are returning an error
 * code that the application may but need not test. Refer to
 * {@link ArmTransactionFactory}and the ARM 4.0 Java Binding Specification for
 * more information about handling errors. <p/>Note that there are some changes
 * from ARM 3.0:
 * <ul>
 * <li>Descriptive metadata has been moved from <code>ArmTransaction</code>
 * to <code>ArmApplicationDefinition</code> and
 * <code>ArmTransactionDefinition</code>.</li>
 * <li>Three methods that were in ARM 3.0 are removed. These methods were
 * almost never used. They also placed restrictions on implementations that
 * could be unacceptable because they required an ARM implementation to return a
 * value that had not been provided by the application. They can be contrasted
 * with other getter methods that return data previously set by the application
 * using ARM: <code>getRespTime()</code>,<code>getStopTime()</code>, and
 * <code>getTranHandle()</code>.</li>
 * <li>The return type for <code>stop()</code> is changed from
 * <code>long</code> to <code>int</code>. The <code>long</code> returned
 * in ARM 3.0 was the response time, and the requirement to return it is
 * considered unacceptable. The <code>int</code> return value is an error
 * indicator (if it's negative).</li>
 * </ul>
 * <p/>Objects implementing this interface are created using
 * {@link ArmTransactionFactory#newArmTransaction}.
 * 
 * @author ARM Working Group of The Open Group
 */
public interface ArmTransaction extends ArmInterface {

	// No Public Constructors

	// Public Instance Methods

	/**
	 * indicates current thread executing on behalf of this transaction.
	 * This method can be called from any thread to indicate that 
	 * the thread is executing on behalf of the transaction instance. 
	 * This is useful when multiple threads execute the same logical 
	 * (ARM) transaction, because instrumentation of resource consumption 
	 * at the thread level can be more precise. The thread remains bound 
	 * to this transaction until {@link #unbindThread} is executed in 
	 * this thread or <code>stop()</code> or <code>reset()</code> is 
	 * executed.
	 * @return 0 on sucess; otherwise, a non-zero error code is returned (as
	 *         specified in 
	 *         {@link org.opengroup.arm40.transaction.ArmInterface}).
	 */
	public int bindThread();

	/**
	 * indicates that the transaction instance is blocked.
	 * Blocked in this context means that the transaction is waiting on 
	 * an external transaction (which may or may not be instrumented 
	 * with ARM) or some other event to complete. It has been found 
	 * useful to separate out this "blocked" time from the elapsed time 
	 * between the <code>start()</code> and <code>stop()</code>. 
	 * {@link #unblocked(long)} indicates when the blocking 
	 * condition has ended. 
	 * A transaction may be blocked by multiple conditions 
	 * simultaneously. A "block handle" returned by block() is 
	 * the input parameter to <code>unblocked()</code> to indicate which 
	 * blocking condition has ended.
	 * @return handle to be passed to a matching <code>unblocked()</code>
	 *         method call.
	 */
	public long blocked();

	/**
	 * gets the contaning application instance.
	 * returns the value passed to the 
	 * <code>newArmTransaction()</code> method of 
	 * <code>ArmTransactionFactory</code>.
	 * @return the containing <code>ArmApplication</code>.
	 */
	public ArmApplication getApplication();

	/**
	 * gets the URI context value.
	 * See the description of {@link #getContextURIValue()}.
	 * @return the URI context value, or <code>null</code>.
	 */
	public String getContextURIValue();

	/**
	 * gets a context property value.
	 * See the description of {@link #setContextValue(int, String)}.
	 * @param index index into the context properties array.
	 * @return the context value at index <code>index</code>, or 
	 *         <code>null</code>.
	 */
	public String getContextValue(int index);

	/**
	 * returns a reference to the correlator for the current transaction. 
	 * It may be a newly created object. It can be executed anytime after 
	 * <code>start()</code> is executed. Each time it is executed, it 
	 * will return the same value until the next <code>stop()</code> or 
	 * <code>reset()</code> is executed. If it is executed at any other 
	 * time, it will return an <code>ArmCorrelator</code> object, but the 
	 * data within the ArmCorrelator object is undefined and should not 
	 * be used.
	 * @return a correlator object. Validity of its content depends on 
	 *         the context of execution, see method description.
	 */
	public ArmCorrelator getCorrelator();

	/**
	 * returns the parent correlator, if set for this transaction.
	 * Returns the last value set on a <code>start()</code> method. 
	 * If no value was set on the <code>start()</code> method, or if 
	 * <code>start()</code> has never executed, it returns 
	 * <code>null</code>.
	 * @return the parent correlator, or <code>null</code>.
	 */
	public ArmCorrelator getParentCorrelator();

	/**
	 * returns the last status value set on a <code>stop()</code> method. 
	 * If <code>stop()</code> has never executed, it returns 
	 * <code>STATUS_INVALID</code>.
	 * @return one of the status values defined in {@link ArmConstants}. 
	 */
	public int getStatus();

	/**
	 * gets the definition metadata for this transaction.
	 * returns the value passed to the 
	 * <code>newArmTransaction()</code> method of 
	 * <code>ArmTransactionFactory</code>.
	 * @return the <code>ArmTransactionDefinition</code> metadata.
	 */
	public ArmTransactionDefinition getDefinition();

	/**
	 * returns the <code>ArmUser</code> currently asociated with this
	 * transaction instance. 
	 * See description of {@link #setUser(ArmUser)}.
	 * @return an <code>ArmUser</code>, or <code>null</code>.
	 */
	public ArmUser getUser();

	/**
	 * gets the current trace request state.
	 * The initial trace request state is <code>false</code>.
	 * See description of {@link #setTraceRequested(boolean)}.
	 * @return the current trace request state. 
	 */
	public boolean isTraceRequested();

	/**
	 * Resets a transaction if it is currently executing.
	 * <p/>
	 * This can be executed at any time. If a transaction is currently
	 * executing [<code>start()</code> executed without a matching 
	 * <code>stop()</code>], the current transaction is discarded and 
	 * treated as if the <code>start()</code> never executed. If
	 * no transaction is currently executing, the state of the object is
	 * unchanged. If there is any doubt about the state of an object, 
	 * <code>reset()</code> gets the object into a known state in which a 
	 * <code>start()</code> may be executed.
	 * <code>reset()</code> clears the arrival time and the current 
	 * correlator; it does not change <code>traceRequested</code> or any 
	 * of the context URI, context values, or user.
	 * 
	 * @return 0 on sucess; otherwise, a non-zero error code is returned (as
	 *         specified in 
	 *         {@link org.opengroup.arm40.transaction.ArmInterface}).
	 */
	public int reset();

	/**
	 * sets the actual transaction start time for the next 
	 * <code>start()</code>.
	 * This method can be used in situations in which the context of a 
	 * transaction is not known when the transaction begins to execute, 
	 * and for which there is a non-trivial delay before the context is 
	 * known. ARM requires that the full context of a transaction be 
	 * known when <code>start()</code> is executed (because the correlator 
	 * is generated at this time). In ARM 2.0 and 3.0 there is no way 
	 * to capture any time spent processing the transaction before the 
	 * context is known. ARM 4.0 introduces the concept of an 
	 * "arrival time". The "arrival time" is when processing of the 
	 * transaction commenced. By default it is the moment in time 
	 * when <code>start()</code> executes. If the delay between the start 
	 * of processing and the execution of <code>start()</code> is 
	 * significant, the application can capture the arrival time by 
	 * invoking <code>setArrivalTime()</code>. This establishes a timestamp 
	 * that will be used at the next <code>start()</code>, after which 
	 * the value will be reset within the <code>ArmTransaction</code>
	 *  object. The <code>reset()</code> and <code>stop()</code> methods 
	 * also clear the value.
	 * @return 0 on sucess; otherwise, a non-zero error code is returned (as
	 *         specified in 
	 *         {@link org.opengroup.arm40.transaction.ArmInterface}).
	 */
	public int setArrivalTime();

	/**
	 * sets the URI context value. 
	 * {@link #getContextURIValue()} returns the value. 
	 * In most scenarios, a URI would be used as a transaction identity 
	 * property or a context property, but not both. The only allowed 
	 * exception is when the base part of the URI is used as an identity 
	 * property, and the full URI (e.g., with the parameters) is used 
	 * as a context property. Any other use of URIs as both identity 
	 * and context properties is invalid.
	 * @param value the URI context value.
	 * @return 0 on sucess; otherwise, a non-zero error code is returned (as
	 *         specified in 
	 *         {@link org.opengroup.arm40.transaction.ArmInterface}).
	 */
	public int setContextURIValue(String value);

	/**
	 * sets a context property value.
	 * This method sets one of the maximum 20 context properties that 
	 * may change for each transaction instance. {@link #getContextValue(int)}
	 * returns the value. The "name" part is available via 
	 * <code>getDefinition().getIdentityProperties().getContextName()</code>.
	 * The values are position-sensitive - they match the position in the 
	 * referenced context name array (see the discussion at 
	 * {@link ArmIdentityProperties} for more details). The context property 
	 * name at the specified array index must have been set to a 
	 * non-<code>null</code> value when the 
	 * <code>ArmTransactionDefinition</code> object was created. 
	 * If the name is <code>null</code> or a zero-length string, both the name
	 * and value are ignored. If the value is <code>null</code> or a 
	 * zero-length string, the meaning is that there is no value for 
	 * this instance. The value should not contain trailing blank characters 
	 * or consist of only blank characters.
	 * @param index index into the context properties array.
	 * @param value the new context property value.
	 * @return 0 on sucess; otherwise, a non-zero error code is returned (as
	 *         specified in 
	 *         {@link org.opengroup.arm40.transaction.ArmInterface}).
	 */
	public int setContextValue(int index, String value);

	/**
	 * Toggles request for tracing this transaction. 
	 * This method is used to suggest or withdraw a suggestion from an 
	 * application that a transaction be traced. 
	 * {@link #isTraceRequested} is used to query the current trace 
	 * request state. The initial state is <code>false</code>. 
	 * Once set, it remains in that state until set to a different state.
	 * @param traceState trace request state.
	 * @return 0 on sucess; otherwise, a non-zero error code is returned (as
	 *         specified in 
	 *         {@link org.opengroup.arm40.transaction.ArmInterface}).
	 */
	public int setTraceRequested(boolean traceState);

	/**
	 * associates a user to the <code>ArmTransaction</code> instance. 
	 * This user, represented by an instance of {@link ArmUser}, 
	 * is assumed to be the user for all <code>start()</code>/
	 * <code>stop()</code> pairs until the association is changed or 
	 * cleared.
	 * <p/>
	 * {@link #getUser()} returns the last value that was set.
	 * @param user the user to be associated with this transaction 
	 *         instance. When <code>null</code>, clears any existing 
	 *         association to an <code>ArmUser</code>
	 * @return 0 on sucess; otherwise, a non-zero error code is returned (as
	 *         specified in 
	 *         {@link org.opengroup.arm40.transaction.ArmInterface}).
	 */
	public int setUser(ArmUser user);

	/**
	 * indicates when a transaction begins.
	 * <p>
	 * Because the response time depends on when <code>start()</code>
	 * executes, it should execute as close to the actual start time as
	 * possible. After <code>start()</code> executes, it should not be
	 * executed again until <code>reset()</code> or <code>stop()</code> is
	 * executed. If <code>start()</code> executes consecutively, the 
	 * behavior is undefined.
	 * <p/>There are four versions of <code>start()</code>, depending on
	 * whether a parent correlator is provided, and if one is provided, the
	 * format of the input data. The length of the correlator is in the first
	 * two bytes of the correlator byte array, with the bytes in network byte
	 * order. When the input is a byte array, the length of the array does not
	 * matter, as long as it is at least long enough to hold the correlator,
	 * based on the two-byte length field.
	 * 
	 * @return 0 on sucess; otherwise, a non-zero error code is returned (as
	 *         specified in 
	 *         {@link org.opengroup.arm40.transaction.ArmInterface}).
	 */
	public int start();

	/**
	 * indicates when a transaction begins. See the description of
	 * {@link #start()}.
	 * 
	 * @param parentCorr
	 *            a parent correlator for this transaction, represented as an
	 *            array of bytes.
	 * @return 0 on sucess; otherwise, a non-zero error code is returned (as
	 *         specified in 
	 *         {@link org.opengroup.arm40.transaction.ArmInterface}).
	 */
	public int start(byte[] parentCorr);

	/**
	 * indicates when a transaction begins. See the description of
	 * {@link #start()}.
	 * 
	 * @param parentCorr
	 *            a parent correlator for this transaction, represented as an
	 *            array of bytes.
	 * @param offset
	 *            the offset into byte array <code>parentCorr</code> pointing
	 *            at the start of the acual correlator data.
	 * @return 0 on sucess; otherwise, a non-zero error code is returned (as
	 *         specified in 
	 *         {@link org.opengroup.arm40.transaction.ArmInterface}).
	 */
	public int start(byte[] parentCorr, int offset);

	/**
	 * indicates when a transaction begins. See the description of
	 * {@link #start()}.
	 * 
	 * @param parentCorr
	 *            a parent correlator object for this transaction.
	 * @return 0 on sucess; otherwise, a non-zero error code is returned (as
	 *         specified in 
	 *         {@link org.opengroup.arm40.transaction.ArmInterface}).
	 */
	public int start(ArmCorrelator parentCorr);

	/**
	 * indicates when a transaction ends and what the status of the transaction
	 * was. Because the response time depends on when <code>stop()</code>
	 * executes, it should execute as close to the actual stop time as 
	 * possible.
	 * If <code>stop()</code> is erroneously issued when there is no
	 * transaction active [<code>start()</code> issued without a matching
	 * <code>stop()</code>], it is ignored.
	 * 
	 * @param status
	 *            one of STATUS_ABORT, STATUS_FAILED, STATUS_GOOD, or
	 *            STATUS_UNKNOWN (all defined in {@link ArmConstants}).
	 * @return 0 on sucess; otherwise, a non-zero error code is returned (as
	 *         specified in 
	 *         {@link org.opengroup.arm40.transaction.ArmInterface}).
	 */
	public int stop(int status);

	/**
	 * indicates when a transaction ends and what the status of the transaction
	 * was. This is an optional form of <code>stop()</code> as a way for an
	 * application to provide additional diagnostic details when the status is
	 * something other than <code>STATUS_GOOD</code>. See the desription of
	 * {@link #stop(int)}.
	 * 
	 * @param status
	 *            one of STATUS_ABORT, STATUS_FAILED, STATUS_GOOD, or
	 *            STATUS_UNKNOWN (all defined in {@link ArmConstants}).
	 * @param diagnosticDetail
	 *            string with additional diagnostic details provided by the
	 *            application
	 * @return 0 on sucess; otherwise, a non-zero error code is returned (as
	 *         specified in 
	 *         {@link org.opengroup.arm40.transaction.ArmInterface}).
	 */
	public int stop(int status, String diagnosticDetail);

	/** indicates current thread not executing on behalf of this 
	 * transaction any more.
	 * See decription of {@link #bindThread()}.
	 * @return 0 on sucess; otherwise, a non-zero error code is returned (as
	 *         specified in 
	 *         {@link org.opengroup.arm40.transaction.ArmInterface}).
	 */
	public int unbindThread();

	/**
	 * indicates that the transaction instance is not blocked any more.
	 * See the description of {@link #blocked()}. 
	 * @param blockHandle handle returned from a previous 
	 *        <code>blocked()</code> method call.
	 * @return 0 on sucess; otherwise, a non-zero error code is returned (as
	 *         specified in 
	 *         {@link org.opengroup.arm40.transaction.ArmInterface}).
	 */
	public int unblocked(long blockHandle);

	/**
	 * provides heartbeat and/or metric value update functionality. <p/>After a
	 * <code>start()</code> there can be any number of <code>update()</code>
	 * calls until a <code>stop()</code>. If it is executed at any other
	 * time, it is ignored. The behavior of <code>update()</code> issued at
	 * any other time is undefined. It is used for two purposes:
	 * <ul>
	 * <li>It serves as a heartbeat to show that a transaction is still
	 * executing. This is especially useful if the transaction is a long-running
	 * job.</li>
	 * <li>When used with
	 * {@link org.opengroup.arm40.metric.ArmTransactionWithMetrics}, a subclass
	 * of <code>ArmTransaction</code>, any of the metric values can be
	 * provided with an <code>update()</code>.</li>
	 * </ul>
	 * 
	 * @return 0 on sucess; otherwise, a non-zero error code is returned (as
	 *         specified in 
	 *         {@link org.opengroup.arm40.transaction.ArmInterface}).
	 */
	public int update();
}