/* ------------------------------------------------------------------------- */
/*                                                                           */
/* Copyright (c) 2003 The Open Group                                         */
/*                                                                           */
/* Permission is hereby granted, free of charge, to any person obtaining a   */
/* copy of this software (the "Software"), to deal in the Software without   */
/* restriction, including without limitation the rights to use, copy,        */
/* modify, merge, publish, distribute, sublicense, and/or sell copies of     */
/* the Software, and to permit persons to whom the Software is furnished     */
/* to do so, subject to the following conditions:                            */
/*                                                                           */
/* The above copyright notice and this permission notice shall be included   */
/* in all copies or substantial portions of the Software.                    */
/*                                                                           */
/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS   */
/* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                */
/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.    */
/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY      */
/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT */
/* OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR  */
/* THE USE OR OTHER DEALINGS IN THE SOFTWARE.                                */
/*                                                                           */
/* ------------------------------------------------------------------------- */
/*                                                                           */
/* File revision information                                                 */
/*                                                                           */
/* $Source: /mnt/cvslocal/sdk4/java/org/opengroup/arm40/transaction/ArmTransactionFactory.java,v $ */
/* $Revision: 1.1 $ */
/* $Date: 2005/03/14 15:14:53 $ */
/*                                                                           */
/* ------------------------------------------------------------------------- */

package org.opengroup.arm40.transaction;

/**
 * Provides methods to create instances of the classes in 
 * the org.opengroup.arm40.transaction package. An error callback method can 
 * be registered for objects created with this factory (see 
 * {@link #setErrorCallback(ArmErrorCallback)}).
 * <p/>
 * Semantics of the factory methods are provided in the description 
 * of the individual methods below.  
 * <p/>
 * 
 * <code>ArmMetricFactory</code> is instantiated using a class loader. 
 * The actual name of the factory implementation class is obtained from the 
 * system property whose name is provided in the 
 * {@link #propertyKey} constant.   
 * <p/>
 * <b>Error Handling Philosophy</b>
 * <p/>
 * If an invalid set of parameters is passed to any method, such as an offset 
 * that extends beyond the end of an array, an object is returned that may 
 * contain dummy data. For example, a <code>null byte[] addressBytes</code> 
 * parameter might result in creating an object with an address of all zeros.
 * Different ARM implementations may handle the situation in different ways,
 * but in all cases, they will return an object that is syntactically 
 * correct; that is, any of its methods can be invoked without causing an 
 * exception, even if the data may be at least partially meaningless. 
 * The <code>getErrorCode()</code> method of each object can be used after 
 * it is created to test whether errors occurred. 
 * See {@link ArmInterface} and the ARM 4.0 Java Binding specification
 * for a more complete explanation.
 * <p/>
 * <code>ArmTransactionFactory</code> also serves as the anchor point for an 
 * application-registered callback function. <code>setErrorCallback()</code>
 * is used to register a callback that will be called if any method of an
 * object created by this factory object sets the error code to a negative 
 * value. The error code is retrieved using <code>getErrorCode()</code>, 
 * defined in <code>ArmInterface</code>, the root for most interfaces in 
 * the ARM specification. The <code>boolean</code> returned by 
 * <code>setErrorCallback()</code> indicates whether the registration is 
 * accepted. If an ARM implementation does not support the callback 
 * function, it will return false. <code>setErrorCallback(null)</code> 
 * unregisters any previously registered callback. Note that due to timing 
 * conditions or specifics of the ARM implementation, a previously 
 * registered callback may continue to be called for an indeterminate 
 * length of time after <code>setErrorCallback(null)</code> is executed. 
 * For a broader discussion of error handling, refer to the ARM 4.0 Java
 * Binding specification.
 * 
 * @author ARM Working Group of The Open Group
 */
public interface ArmTransactionFactory extends ArmInterface {

// Public Constants
    /**
     * name of system property holding the implementation class name.
     */
    public static final String propertyKey = "Arm40.ArmTransactionFactory";

// Public Instance Methods
    /**
     * Creates the <code>ArmApplication</code> object to which 
     * transaction instances are related. 
     * <code>definition</code> is the only required non-<code>null</code>
     * parameter.
     * 
     * @see ArmApplication
     * @param definition metadata describing the type of the ARM application.
     *                   Must not be <code>null</code>.
     * @param group Name of the group the application belongs to.
     *              May be <code>null</code>.
     * @param instance Name this application instance is given.
     *                 May be <code>null</code>.
     * @param contextValues An array of strings providing the value parts of
     *                      (name,value) context properties.
     *                      May be <code>null</code>.
     * @return a new <code>ArmApplication</code> instance.
     */
    public ArmApplication newArmApplication(
        ArmApplicationDefinition definition, String group, 
        String instance, String[] contextValues);
        
    /**
     * Creates the <code>ArmApplicationDefinition</code> object that 
     * describes the metadata about an application.
     * That is, the descriptive data that is the same for all 
     * instances of the same application. 
     * <code>name</code> is the only required non-<code>null</code> parameter.
     * 
     * @see ArmApplicationDefinition
     * @param name the name of the application type. A name should be chosen 
     *             that is unique.
     *             Must not be <code>null</code>
     * @param identityProperties a set of properties contibuting to the 
     *                           identity of the application definition. 
     *                           May be <code>null</code>. 
     * 							 See {@link #newArmIdentityProperties}. 
     * @param id An optional 16-byte ID associated with the identity of 
     *           the application definition.
     * @return a new <code>ArmApplicationDefinition</code>.
     */
    public ArmApplicationDefinition newArmApplicationDefinition(
        String name, ArmIdentityProperties identityProperties, ArmID id);
        
    /**
     * Creates a correlator from a byte array in the correct format. 
     * No length field is passed to this method because ARM requires that 
     * the length of the correlator be found in the first two bytes 
     * (in network byte order) of the byte array. 
     * The correlator must be no longer than 
     * <code>ArmConstants.ARM_CORR_MAX_LENGTH</code>. 
     * If the correlator is longer than the ARM implementation supports, 
     * an <code>ArmCorrelator</code> object will be created, but 
     * it may contain dummy data, at the discretion of the ARM implementation.
     * 
     * @see ArmCorrelator
     * @param corrBytes a byte array containig correlator data. 
     * @return a new <code>ArmCorrelator</code>.
     */
    public ArmCorrelator newArmCorrelator(byte[] corrBytes);
    
    /**
     * Creates a correlator from a byte array in the correct format.
     * See description of {@link #newArmCorrelator(byte[])}. Since
     * the start of the correlator bytes within the array is at
     * position <code>offset</code>, the length of the correlator is taken
     * from the bytes at <code>offset</code> and <code>offset+1</code>.
     * 
     * @param corrBytes a byte array containig correlator data.
     * @param offset start of the correlator bytes within the array.
     * @return a new <code>ArmCorrelator</code>.
     */
    public ArmCorrelator newArmCorrelator(byte[] corrBytes, int offset);
    
    /**
     * Creates the objects that contain an immutable 16-byte ID.
     * Having 16-byte IDs accommodates some widely used IDs, such 
     * as the UUID (universally unique identifier) defined in the DCE 
     * and IETF standards.
     * @see ArmID
     * 
     * @param idBytes a byte array containig ID data.
     * @return a new <code>ArmID</code>.
     */
    public ArmID newArmID(byte[] idBytes);
    
    /**
     * Creates the objects that contain an immutable 16-byte ID.
     * See description of {@link #newArmID(byte[])}.
     * 
     * @param idBytes a byte array containig ID data.
     * @param offset start of the ID bytes within the array.
     * @return a new <code>ArmID</code>.
     */
    public ArmID newArmID(byte[] idBytes, int offset);
    
    /**
     * Creates an object that contains an immutable set of identity 
     * property names and values, and an immutable set of context property 
     * names. 
     * These properties describe identity properties that are common to 
     * all instances of an application or transaction.
     * <p/>
     * The names and values are provided in arrays of strings. 
     * There can be up to twenty elements in each of the arrays. A 
     * <code>null</code> reference and a zero-length string are both 
     * treated as a null element. It is permissible to have null elements 
     * in the middle of the array. For example, it is permissible for the 
     * elements at indices 0 and 19 to be non-<code>null</code> and all 
     * the elements from indices 1 to 18 to be <code>null</code>. 
     * The arrays are position sensitive.
     * <p/>
     * The identity property name and value arrays should contain the same
     * number of elements at the same array indices; for each 
     * non-<code>null</code> name or value, there should be a corresponding 
     * non-<code>null</code> value or name, respectively, at the same array
     * index. For any array index, if either the name or the value is 
     * <code>null</code>, both the name and the value are ignored.
     * <p/>
     * The context property names are provided in this object. The 
     * context values are provided in an <code>ArmApplication</code>, 
     * <code>ArmTransaction</code> or <code>ArmTranReport</code> object.
     * If the name at an array index is <code>null</code>, the 
     * corresponding value in those objects is ignored.
     * 
     * @see ArmIdentityProperties 
     * @param identityNames an array of identity property names.
     * @param identityValues an array of identity property values.
     * @param contextNames an array of identity property names.
     * @return a new <ArmIdentityProperties> object.
     */
    public ArmIdentityProperties newArmIdentityProperties(
        String[] identityNames, String[] identityValues, 
        String[] contextNames);
        
    /**
     * Creates an object that extends <code>newArmIdentityProperties</code>
     * to also include a URI property, which is also immutable.
     * See the {@link #newArmIdentityProperties} description for all 
     * other parameters.
     * 
     * @see ArmIdentityPropertiesTransaction 
     * @param identityNames an array of identity property names.
     * @param identityValues an array of identity property values.
     * @param contextNames an array of identity property names.
     * @param uriValue the URI property value.
     * @return a new <ArmIdentityPropertiesTransaction> object.
     */
    public ArmIdentityPropertiesTransaction 
        newArmIdentityPropertiesTransaction(
            String[] identityNames, String[] identityValues, 
            String[] contextNames, String uriValue);
            
    /**
     * Creates an object that represents an instance of a transaction.
     * 
     * @see ArmTransaction
     * @param app the application instance the transaction belongs to.
     * @param definition the metadata describing the type of the transaction.
     * @return a new <code>ArmTransaction</code> instance.
     */
    public ArmTransaction newArmTransaction(ArmApplication app, 
        ArmTransactionDefinition definition);
        
    /**
     * Creates an object that represents the metadata about a transaction.
     * 
     * @see ArmTransactionDefinition
     * @param app the metadata describing the type of the containing 
     *            application.
     *            Must not be <code>null</code>.
     * @param name the name of the transaction type. A name should be chosen 
     *             that is unique. 
     *             Must not be <code>null</code>.
     * @param identityProperties a set of properties contibuting to the 
     *                           identity of the transaction definition. 
     *                           May be <code>null</code>. 
     * 							 See {@link #newArmIdentityProperties}.
     * @param id An optional 16-byte ID associated with the identity of 
     *           the transaction definition.
     * @return a new <code>ArmTransactionDefinition</code>.
     */
    public ArmTransactionDefinition newArmTransactionDefinition(
	    ArmApplicationDefinition app, String name, 
        ArmIdentityPropertiesTransaction identityProperties, ArmID id);
        
    /**
     * Creates an ArmUser object that represents the user who invoked 
     * (directly or indirectly) the transaction.
     * 
     * @see ArmUser
     * @param name name of the user.
     * @param id An optional 16-byte ID associated with the user.
     * @return a new <code>ArmUser</code> object.
     */
    public ArmUser newArmUser(String name, ArmID id);
    
    /**
     * Registers an error callback for objects created through this factory.
     * See the error handling philosophy comment in the description of
     * this interface.
     * 
     * @param errorCallback an application object implementing the
     *                      <code>ArmErrorCallback> interface.
     * @return <code>true</code> if registration is accepted.
     */
    public boolean setErrorCallback(ArmErrorCallback errorCallback);
}
